#version 300 es
// 浮点数的精度
precision highp float;

/**
通用
*/
uniform float fAlpha;
uniform int fPlayLast;
uniform int fTypeS;
uniform int fLastType;


/**
点数据类型相关变量
*/
in vec4 vColor;


/**
图片类型相关变量方法
*/
//2D采样器，可以理解为图片的唯一标识
uniform sampler2D uTextureUnitU;
uniform sampler2D uTextureUnitV;
uniform sampler2D uTextureColor;
uniform float fMinU;
uniform float fMaxU;
uniform float fMaxV;
uniform float fMinV;

uniform sampler2D uLastTextureUnitU;
uniform sampler2D uLastTextureUnitV;
uniform sampler2D uLastTextureColor;
uniform float fLastMinU;
uniform float fLastMaxU;
uniform float fLastMaxV;
uniform float fLastMinV;
const float stand = 1.0/256.0;
const float maxC = 1.0-stand/2.0;
//图片的坐标点 固定为
// 0.0f, 0.0f,
// 0.0f, 1.0f,
// 1.0f, 0.0f,
// 1.0f, 1.0f
in vec2 vTexCoord;

//图片标值，色值转换相关方法
float convertR(vec2, sampler2D, float, float);
float getRealValue(vec2, sampler2D, float, float);
vec4 getColorByType(float, int, sampler2D, float);
vec4 getColorWind(float, sampler2D, float);
vec4 getColorTemp(float, sampler2D, float);
vec4 getColorPressure(float, sampler2D, float);
vec4 getColorRainDay(float, sampler2D, float);
vec4 getColorAqi(float, sampler2D, float);

vec4 getFinalColor(vec2, sampler2D, sampler2D, sampler2D, int, float, float, float, float, float);

out vec4 fragColor;
void main(){
    vec4 color = getFinalColor(vTexCoord, uTextureUnitU, uTextureUnitV, uTextureColor, fTypeS, fMaxU, fMinU, fMaxV, fMinV, fAlpha);
    if (fPlayLast==1){
        vec4 lastColor = getFinalColor(vTexCoord, uLastTextureUnitU, uLastTextureUnitV, uLastTextureColor, fLastType, fLastMaxU, fLastMinU, fLastMaxV, fLastMinV, (1.0-fAlpha));
        fragColor = color.a*color+(1.0-color.a)*lastColor;
    } else {
        fragColor = color;
    }
}

//计算颜色
vec4 getFinalColor(vec2 texCoord, sampler2D textureUnitU, sampler2D textureUnitV, sampler2D textureColor, int type, float maxU, float minU, float maxV, float minV, float alpha){
    if (type==3){
        //下雨类型
        return vColor;
    } else {
        //其他类型
        float rU = convertR(texCoord, textureUnitU, maxU, minU);
        float r;
        //风力类型，算两个U/V两个方向的值
        if (type==0){
            float rV = convertR(texCoord, textureUnitV, maxV, minV);
            r= sqrt(pow(rU, 2.0)+pow(rV, 2.0));
        } else {
            r = rU;
        }
        return getColorByType(r, type, textureColor, alpha);
    }
}

/**
图片类方法开始
*/

vec4 getColorByType(float r, int type, sampler2D textureColor, float alpha){
    if (type==0){
        return getColorWind(r, textureColor, alpha);
    } else if (type ==1){
        return getColorTemp(r, textureColor, alpha);
    } else if (type == 2){
        return getColorPressure(r, textureColor, alpha);
    } else if (type == 5){
        return getColorAqi(r, textureColor, alpha);
    } else {
        return getColorRainDay(r, textureColor, alpha);
    }
}

const float minAQIValue = 0.0;
const float maxAQIValue = 500.0;
vec4 getColorAqi(float r, sampler2D textureColor, float alpha){
    if (r < 1.0) {
        return vec4(0.0, 0.0, 0.0, 0.0);
    } else {
        float c = (r - minAQIValue) / (maxAQIValue - minAQIValue);
        //texture2D获取到指定纹理图片中指定位置的一个像素
        vec4 color = texture(textureColor, vec2(c, 0.5));
        color.a = color.a*alpha;
        return color;
    }
}

vec4 getColorWind(float r, sampler2D textureColor, float alpha){
    float c;
    if (r<0.0){
        c = 0.0;
    } else if (r<1.0){
        c =r*0.07142857;
    } else if (r<3.0){
        c =(r-1.0)*0.03571428+0.07142857;
    } else if (r<5.0){
        c =(r-3.0)*0.03571428+0.14285714;
    } else if (r<75.0){
        c =(r-5.0)*0.01020408+0.21428571;
    } else if (r<105.0){
        c =(r-75.0)*0.00238095+0.92857143;
    } else {
        c = 1.0;
    }
    //texture2D获取到指定纹理图片中指定位置的一个像素
    vec4 color = texture(textureColor, vec2(c, 0.5));
    color.a = color.a*alpha;
    return color;
}
vec4 getColorTemp(float r, sampler2D textureColor, float alpha){
    float c = r<-70.0?0.0:r>50.0?1.0:((r+70.0)/120.0);
    vec4 color = texture(textureColor, vec2(c, 0.5));
    //    vec4 color = texture(uTextureColor, vec2(c,0.5));
    color.a = color.a*alpha;
    return color;
}
vec4 getColorPressure(float r, sampler2D textureColor, float alpha){
    float c;
    if (r<870.0){
        c = 0.0;
    } else if (r<920.0){
        c =(r-870.0)*0.00117647;
    } else if (r<948.0){
        c =(r-920.0)*0.00210084+0.05882352;
    } else if (r<964.0){ //2
        c =(r-948.0)*0.00735294+0.11764706;
    } else if (r<973.0){
        c =(r-964.0)*0.00653595+0.23529408;
    } else if (r<1021.0){ //6
        c =(r-973.0)*0.00735294+0.2941176;
    } else if (r<1030.0){
        c =(r-1021.0)*0.00653595+0.64705872;
    } else if (r<1080.0){ //5
        c =(r-1030.0)*0.00588235+0.70588224;
    } else {
        c = 1.0;
    }
    vec4 color = texture(textureColor, vec2(c, 0.5));
    color.a = color.a*alpha;
    return color;
}


vec4 getColorRainDay(float r, sampler2D textureColor, float alpha){
    float c;
    if (r<0.1){
        c = 0.0;
    } else if (r<0.2){
        //0.1-0.2 ,占1/13份额
        c =(r-0.1)*0.7692308;
    } else if (r<0.6){
        //0.2-0.6 ,占1/13份额
        c =(r-0.2)*0.1923077+0.07692308;
    } else if (r<1.2){
        c =(r-0.6)*0.12820513+0.15384615;
    } else if (r<4.0){
        c =(r-1.2)*0.02747253+0.23076924;
    } else if (r<8.0){
        c =(r-4.0)*0.01923077+0.30769232;
    } else if (r<10.0){
        c =(r-8.0)*0.03846154+0.3846154;
    } else if (r<45.0){
        //1050-1080 ,占8/14份额  /40*8/14+6/14
        c =(r-10.0)*0.01538462+0.46153844;
    } else {
        c = 1.0;
    }
    vec4 color = texture(textureColor, vec2(c, 0.5));
    if (c!=0.0){
        if (r<0.2){
            alpha = alpha*0.5;
        } else if (r<0.6){
            alpha = alpha*0.6;
        } else if (r<4.0){
            alpha = alpha*0.8;
        }
        color.a = color.a*alpha;
    } else {
        color.a = 0.0;
    }
    return color;
}

float convertR(vec2 uv, sampler2D texture, float max, float min) {
    vec2 px = vec2(stand, stand);//(1/256, 1/256)
    vec2 uvt = uv * vec2(256.0, 256.0);//纹理坐标(0~1,0~1)*(256.0,256.0)
    vec2 vc = (floor(uvt)) * px;//floor(x)向下取整 纹理坐标*(256.0,256.0)*(1/256,1/256)
    vec2 f = fract(uvt);//fract(x) 返回x的小数部分  纹理坐标*(256.0,256.0)的小数部分

    //取四个点的值(当前点、右侧点、下侧点、右下侧点)，然后做插值(插值(插值(当前点，右侧点),插值(下侧点，右下侧点)))
    float tl = getRealValue(vc, texture, max, min);
    float tr = getRealValue(vc + vec2(px.x, 0), texture, max, min);
    float bl = getRealValue(vc + vec2(0, px.y), texture, max, min);
    //    vec2 vc = (uv * vec2(255.0, 255.0))/ vec2(257.0, 257.0);
    float br = getRealValue(vc + px, texture, max, min);
    //mix线性插值 dest=x*(1-level)+y*level 假设x=1,y=2,level=0.5，dest=1*0.5+2*0.5=1.5，level越大其结果越趋向于y
    return mix(mix(tl, tr, f.x), mix(bl, br, f.x), f.y);
    //    float tl = getRealValue(uv, texture, max, min);
    //    return br;
}

float getRealValue(vec2 uv, sampler2D u_screen, float max, float min) {
    //    if (uv.x>maxC)uv.x = maxC;
    //    if (uv.y>maxC)uv.y = maxC;
    vec4 color=texture(u_screen, uv);
    float r=color.r;//因为灰度图的rgb三个颜色分量相等，所以可以任意取出一个颜色通道的值作为灰度值，取值范围0~1
    float  result =r*(max-min)+min;
    return result;
}

/**
图片类方法结束
*/